@@ -1,7 +1,7 @@
 /*
  * arch/arm/mach-tegra/board-enterprise-sensors.c
  *
- * Copyright (c) 2011, NVIDIA CORPORATION, All rights reserved.
+ * Copyright (c) 2011-2012, NVIDIA CORPORATION, All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -101,6 +101,7 @@ static void nct1008_probe_callback(struct nct1008_data *data)
 
 	thermal_device->name = "nct1008";
 	thermal_device->data = data;
+	thermal_device->id = THERMAL_DEVICE_ID_NCT_EXT;
 	thermal_device->offset = TDIODE_OFFSET;
 	thermal_device->get_temp = nct_get_temp;
 	thermal_device->get_temp_low = nct_get_temp_low;
@@ -108,7 +109,7 @@ static void nct1008_probe_callback(struct nct1008_data *data)
 	thermal_device->set_alert = nct_set_alert;
 	thermal_device->set_shutdown_temp = nct_set_shutdown_temp;
 
-	tegra_thermal_set_device(thermal_device);
+	tegra_thermal_device_register(thermal_device);
 }
 
 static struct nct1008_platform_data enterprise_nct1008_pdata = {
@@ -131,7 +132,6 @@ static void enterprise_nct1008_init(void)
 {
 	int ret;
 
-	tegra_gpio_enable(TEGRA_GPIO_PH7);
 	ret = gpio_request(TEGRA_GPIO_PH7, "temp_alert");
 	if (ret < 0) {
 		pr_err("%s: gpio_request failed %d\n", __func__, ret);
@@ -213,7 +213,6 @@ static void mpuirq_init(void)
 #if (MPU_GYRO_TYPE == MPU_TYPE_MPU3050)
 #if	MPU_ACCEL_IRQ_GPIO
 	/* ACCEL-IRQ assignment */
-	tegra_gpio_enable(MPU_ACCEL_IRQ_GPIO);
 	ret = gpio_request(MPU_ACCEL_IRQ_GPIO, MPU_ACCEL_NAME);
 	if (ret < 0) {
 		pr_err("%s: gpio_request failed %d\n", __func__, ret);
@@ -230,7 +229,6 @@ static void mpuirq_init(void)
 #endif
 
 	/* MPU-IRQ assignment */
-	tegra_gpio_enable(MPU_GYRO_IRQ_GPIO);
 	ret = gpio_request(MPU_GYRO_IRQ_GPIO, MPU_GYRO_NAME);
 	if (ret < 0) {
 		pr_err("%s: gpio_request failed %d\n", __func__, ret);
@@ -589,7 +587,6 @@ static int enterprise_cam_init(void)
 		gpio_direction_output(enterprise_cam_gpio_data[i].gpio,
 				      enterprise_cam_gpio_data[i].value);
 		gpio_export(enterprise_cam_gpio_data[i].gpio, false);
-		tegra_gpio_enable(enterprise_cam_gpio_data[i].gpio);
 	}
 
 	tegra_get_board_info(&bi);
