@@ -1,7 +1,7 @@
 /*
  * arch/arm/mach-tegra/board-kai.c
  *
- * Copyright (c) 2012, NVIDIA Corporation.
+ * Copyright (c) 2012, NVIDIA CORPORATION.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -56,31 +56,71 @@
 #include <asm/mach/arch.h>
 #include <mach/usb_phy.h>
 #include <mach/thermal.h>
+#include <mach/tegra_fiq_debugger.h>
 
 #include "board.h"
 #include "clock.h"
 #include "board-kai.h"
+#include "board-touch.h"
 #include "devices.h"
 #include "gpio-names.h"
 #include "fuse.h"
 #include "pm.h"
 #include "wdt-recovery.h"
 
+static struct balanced_throttle throttle_list[] = {
+	{
+		.id = BALANCED_THROTTLE_ID_TJ,
+		.throt_tab_size = 10,
+		.throt_tab = {
+			{      0, 1000 },
+			{ 640000, 1000 },
+			{ 640000, 1000 },
+			{ 640000, 1000 },
+			{ 640000, 1000 },
+			{ 640000, 1000 },
+			{ 760000, 1000 },
+			{ 760000, 1050 },
+			{1000000, 1050 },
+			{1000000, 1100 },
+		},
+	},
+#ifdef CONFIG_TEGRA_SKIN_THROTTLE
+	{
+		.id = BALANCED_THROTTLE_ID_SKIN,
+		.throt_tab_size = 6,
+		.throt_tab = {
+			{ 640000, 1200 },
+			{ 640000, 1200 },
+			{ 760000, 1200 },
+			{ 760000, 1200 },
+			{1000000, 1200 },
+			{1000000, 1200 },
+		},
+	},
+#endif
+};
+
 /* All units are in millicelsius */
 static struct tegra_thermal_data thermal_data = {
-	.temp_throttle = 85000,
+	.shutdown_device_id = THERMAL_DEVICE_ID_NCT_EXT,
 	.temp_shutdown = 90000,
-	.temp_offset = TDIODE_OFFSET, /* temps based on tdiode */
+#if defined(CONFIG_TEGRA_EDP_LIMITS) || defined(CONFIG_TEGRA_THERMAL_THROTTLE)
+	.throttle_edp_device_id = THERMAL_DEVICE_ID_NCT_EXT,
+#endif
 #ifdef CONFIG_TEGRA_EDP_LIMITS
 	.edp_offset = TDIODE_OFFSET,  /* edp based on tdiode */
 	.hysteresis_edp = 3000,
 #endif
-#ifdef CONFIG_TEGRA_THERMAL_SYSFS
+#ifdef CONFIG_TEGRA_THERMAL_THROTTLE
+	.temp_throttle = 85000,
 	.tc1 = 0,
 	.tc2 = 1,
 	.passive_delay = 2000,
-#else
-	.hysteresis_throttle = 1000,
+#endif
+#ifdef CONFIG_TEGRA_SKIN_THROTTLE
+	.skin_device_id = THERMAL_DEVICE_ID_SKIN,
+	.temp_throttle_skin = 43000,
 #endif
 };
 
@@ -110,11 +150,16 @@ static noinline void __init kai_bt_st(void)
 
 	platform_device_register(&wl128x_device);
 	platform_device_register(&btwilink_device);
-	tegra_gpio_enable(TEGRA_GPIO_PU0);
 }
 
 static struct resource kai_bluesleep_resources[] = {
 	[0] = {
+		.name = "gpio_host_wake",
+			.start  = TEGRA_GPIO_PU6,
+			.end    = TEGRA_GPIO_PU6,
+			.flags  = IORESOURCE_IO,
+	},
+	[1] = {
 		.name = "host_wake",
 			.start	= TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_PU6),
 			.end	= TEGRA_GPIO_TO_IRQ(TEGRA_GPIO_PU6),
@@ -123,7 +168,7 @@ static struct resource kai_bluesleep_resources[] = {
 };
 
 static struct platform_device kai_bluesleep_device = {
-	.name		= "tibluesleep",
+	.name		= "bluesleep",
 	.id		= 0,
 	.num_resources	= ARRAY_SIZE(kai_bluesleep_resources),
 	.resource	= kai_bluesleep_resources,
@@ -132,7 +177,6 @@ static struct platform_device kai_bluesleep_device = {
 static noinline void __init kai_tegra_setup_tibluesleep(void)
 {
 	platform_device_register(&kai_bluesleep_device);
-	tegra_gpio_enable(TEGRA_GPIO_PU6);
 }
 
 static __initdata struct tegra_clk_init_table kai_clk_init_table[] = {
@@ -260,11 +304,17 @@ static struct regulator_consumer_supply smb349_vbus_supply[] = {
 	REGULATOR_SUPPLY("usb_bat_chg", NULL),
 };
 
+static struct regulator_consumer_supply smb349_otg_vbus_supply[] = {
+	REGULATOR_SUPPLY("usb_vbus_otg", NULL),
+};
+
 static struct smb349_charger_platform_data smb349_charger_pdata = {
 	.max_charge_current_mA = 1000,
 	.charging_term_current_mA = 100,
 	.consumer_supplies = smb349_vbus_supply,
 	.num_consumer_supplies = ARRAY_SIZE(smb349_vbus_supply),
+	.otg_consumer_supplies = smb349_otg_vbus_supply,
+	.num_otg_consumer_supplies = ARRAY_SIZE(smb349_otg_vbus_supply),
 };
 
 static struct i2c_board_info kai_i2c4_smb349_board_info[] = {
@@ -460,7 +510,6 @@ static struct platform_device tegra_camera = {
 };
 
 static struct platform_device *kai_spi_devices[] __initdata = {
-	&tegra_spi_device4,
 	&tegra_spi_device1,
 };
 
@@ -474,11 +523,11 @@ static struct spi_clk_parent spi_parent_clk[] = {
 #endif
 };
 
-static struct tegra_spi_platform_data kai_spi_pdata = {
-	.is_dma_based		= true,
-	.max_dma_buffer		= (16 * 1024),
-	.is_clkon_always	= false,
-	.max_rate		= 100000000,
+static struct tegra_spi_platform_data kai_spi1_pdata = {
+		.is_dma_based           = true,
+		.max_dma_buffer         = (128),
+		.is_clkon_always        = false,
+		.max_rate               = 100000000,
 };
 
 static void __init kai_spi_init(void)
@@ -496,9 +545,10 @@ static void __init kai_spi_init(void)
 		spi_parent_clk[i].parent_clk = c;
 		spi_parent_clk[i].fixed_clk_rate = clk_get_rate(c);
 	}
-	kai_spi_pdata.parent_clk_list = spi_parent_clk;
-	kai_spi_pdata.parent_clk_count = ARRAY_SIZE(spi_parent_clk);
-	tegra_spi_device4.dev.platform_data = &kai_spi_pdata;
+
+	kai_spi1_pdata.parent_clk_list = spi_parent_clk;
+	kai_spi1_pdata.parent_clk_count = ARRAY_SIZE(spi_parent_clk);
+	tegra_spi_device1.dev.platform_data = &kai_spi1_pdata;
 	platform_add_devices(kai_spi_devices,
 				ARRAY_SIZE(kai_spi_devices));
 
@@ -571,7 +621,9 @@ static struct platform_device *kai_devices[] __initdata = {
 #if defined(CONFIG_TEGRA_IOVMM_SMMU) || defined(CONFIG_TEGRA_IOMMU_SMMU)
 	&tegra_smmu_device,
 #endif
-	&tegra_wdt_device,
+	&tegra_wdt0_device,
+	&tegra_wdt1_device,
+	&tegra_wdt2_device,
 #if defined(CONFIG_TEGRA_AVP)
 	&tegra_avp_device,
 #endif
@@ -615,9 +667,6 @@ static int __init kai_touch_init(void)
 {
 	int touch_id;
 
-	tegra_gpio_enable(KAI_TS_ID1);
-	tegra_gpio_enable(KAI_TS_ID2);
-
 	gpio_request(KAI_TS_ID1, "touch-id1");
 	gpio_direction_input(KAI_TS_ID1);
 
@@ -692,7 +741,7 @@ static struct tegra_usb_platform_data tegra_ehci1_utmi_pdata = {
 	.op_mode = TEGRA_USB_OPMODE_HOST,
 	.u_data.host = {
 		.vbus_gpio = -1,
-		.vbus_reg = NULL,
+		.vbus_reg = "usb_vbus_otg",
 		.hot_plug = true,
 		.remote_wakeup_supported = true,
 		.power_off_on_suspend = true,
@@ -719,7 +768,7 @@ static struct tegra_usb_platform_data tegra_ehci2_utmi_pdata = {
 	.u_data.host = {
 		.vbus_gpio = -1,
 		.vbus_reg = NULL,
-		.hot_plug = true,
+		.hot_plug = false,
 		.remote_wakeup_supported = true,
 		.power_off_on_suspend = true,
 
@@ -759,10 +808,6 @@ static void kai_modem_init(void)
 {
 	int ret;
 
-	tegra_gpio_enable(TEGRA_GPIO_W_DISABLE);
-	tegra_gpio_enable(TEGRA_GPIO_MODEM_RSVD1);
-	tegra_gpio_enable(TEGRA_GPIO_MODEM_RSVD2);
-
 	ret = gpio_request(TEGRA_GPIO_W_DISABLE, "w_disable_gpio");
 	if (ret < 0)
 		pr_err("%s: gpio_request failed for gpio %d\n",
@@ -808,16 +853,11 @@ static void kai_audio_init(void)
 	}
 }
 
-static void kai_nfc_init(void)
-{
-	tegra_gpio_enable(TEGRA_GPIO_PX0);
-	tegra_gpio_enable(TEGRA_GPIO_PS7);
-	tegra_gpio_enable(TEGRA_GPIO_PR3);
-}
-
 static void __init tegra_kai_init(void)
 {
-	tegra_thermal_init(&thermal_data);
+	tegra_thermal_init(&thermal_data,
+				throttle_list,
+				ARRAY_SIZE(throttle_list));
 	tegra_clk_init_from_table(kai_clk_init_table);
 	kai_pinmux_init();
 	kai_i2c_init();
@@ -827,7 +867,6 @@ static void __init tegra_kai_init(void)
 	kai_edp_init();
 #endif
 	kai_uart_init();
-	kai_tsensor_init();
 	kai_audio_init();
 	platform_add_devices(kai_devices, ARRAY_SIZE(kai_devices));
 	tegra_ram_console_debug_init();
@@ -837,9 +876,8 @@ static void __init tegra_kai_init(void)
 	kai_touch_init();
 	kai_keys_init();
 	kai_panel_init();
-	kai_bt_st();
 	kai_tegra_setup_tibluesleep();
-	kai_nfc_init();
+	kai_bt_st();
 	kai_sensors_init();
 	kai_pins_state_init();
 	kai_emc_init();
@@ -848,6 +886,7 @@ static void __init tegra_kai_init(void)
 #ifdef CONFIG_TEGRA_WDT_RECOVERY
 	tegra_wdt_recovery_init();
 #endif
+	tegra_serial_debug_init(TEGRA_UARTD_BASE, INT_WDT_CPU, NULL, -1, -1);
 }
 
 static void __init kai_ramconsole_reserve(unsigned long size)
